import logging
from concurrent import futures

import argparse
import logging
import sys
from concurrent import futures

import grpc

from fluxgrpc.protos import flux_pb2_grpc

import fluxgrpc.defaults as defaults


def get_parser():
    parser = argparse.ArgumentParser(
        description="Flux Service Endpoint",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    subparsers = parser.add_subparsers(
        help="actions",
        title="actions",
        description="actions",
        dest="command",
    )
    start = subparsers.add_parser(
        "start",
        description="Start the server.",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    start.add_argument(
        "--workers",
        help=f"Number of workers (defaults to {defaults.workers})",
        default=defaults.workers,
        type=int,
    )
    start.add_argument(
        "--port",
        help=f"Port to run application (defaults to {defaults.port})",
        default=defaults.port,
        type=int,
    )
    start.add_argument(
        "--host",
        help="Host to run application (defaults to localhost)",
        default="localhost",
    )
    return parser


class FluxService(flux_pb2_grpc.FluxServiceServicer):
    """
    A FluxService is a grpc service to run jobs with Flux.
    """
    def SubmitJob(self, request, context):
        """
        Submit a job
        """
        print(request)
        response = flux_pb2_grpc.SubmitResponse()
        print(response)
        return response


def serve(args):
    """
    start the Flux Service
    """
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=args.workers))
    endpoint = FluxService()
    flux_pb2_grpc.add_FluxServiceServicer_to_server(endpoint, server)
    host = f"{args.host}:{args.port}"
    server.add_insecure_port(f"{host}")
    print(f"🥞️ Starting flux service at {host}")
    server.start()
    server.wait_for_termination()


def main():
    """
    Light wrapper main to provide a parser with port/workers
    """
    parser = get_parser()

    # If the user didn't provide any arguments, show the full help
    if len(sys.argv) == 1:
        help()

    # If an error occurs while parsing the arguments, the interpreter will exit with value 2
    args, _ = parser.parse_known_args()
    logging.basicConfig()
    serve(args)


if __name__ == "__main__":
    main()


from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class SubmitRequest(_message.Message):
    __slots__ = ("jobspec",)
    JOBSPEC_FIELD_NUMBER: _ClassVar[int]
    jobspec: str
    def __init__(self, jobspec: _Optional[str] = ...) -> None: ...

class SubmitResponse(_message.Message):
    __slots__ = ("status",)
    class ResultType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        SUBMIT_SUCCESS: _ClassVar[SubmitResponse.ResultType]
        SUBMIT_ERROR: _ClassVar[SubmitResponse.ResultType]
        SUBMIT_DENIED: _ClassVar[SubmitResponse.ResultType]
    SUBMIT_SUCCESS: SubmitResponse.ResultType
    SUBMIT_ERROR: SubmitResponse.ResultType
    SUBMIT_DENIED: SubmitResponse.ResultType
    STATUS_FIELD_NUMBER: _ClassVar[int]
    status: SubmitResponse.ResultType
    def __init__(self, status: _Optional[_Union[SubmitResponse.ResultType, str]] = ...) -> None: ...

class InfoRequest(_message.Message):
    __slots__ = ("jobid",)
    JOBID_FIELD_NUMBER: _ClassVar[int]
    jobid: str
    def __init__(self, jobid: _Optional[str] = ...) -> None: ...

class InfoResponse(_message.Message):
    __slots__ = ("status",)
    class ResultType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        INFO_SUCCESS: _ClassVar[InfoResponse.ResultType]
        INFO_ERROR: _ClassVar[InfoResponse.ResultType]
        INFO_DENIED: _ClassVar[InfoResponse.ResultType]
    INFO_SUCCESS: InfoResponse.ResultType
    INFO_ERROR: InfoResponse.ResultType
    INFO_DENIED: InfoResponse.ResultType
    STATUS_FIELD_NUMBER: _ClassVar[int]
    status: InfoResponse.ResultType
    def __init__(self, status: _Optional[_Union[InfoResponse.ResultType, str]] = ...) -> None: ...

class JobsRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class JobInfo(_message.Message):
    __slots__ = ("jobid", "status")
    JOBID_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    jobid: str
    status: str
    def __init__(self, jobid: _Optional[str] = ..., status: _Optional[str] = ...) -> None: ...

class JobsResponse(_message.Message):
    __slots__ = ("status", "jobs")
    class ResultType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        JOBS_SUCCESS: _ClassVar[JobsResponse.ResultType]
        JOBS_ERROR: _ClassVar[JobsResponse.ResultType]
        JOBS_DENIED: _ClassVar[JobsResponse.ResultType]
    JOBS_SUCCESS: JobsResponse.ResultType
    JOBS_ERROR: JobsResponse.ResultType
    JOBS_DENIED: JobsResponse.ResultType
    STATUS_FIELD_NUMBER: _ClassVar[int]
    JOBS_FIELD_NUMBER: _ClassVar[int]
    status: JobsResponse.ResultType
    jobs: str
    def __init__(self, status: _Optional[_Union[JobsResponse.ResultType, str]] = ..., jobs: _Optional[str] = ...) -> None: ...



import sys
import fluxgrpc.defaults as defaults
import flux
from flux.cli import base
import argparse
from .client import FluxClient

class SubmitCmd(base.SubmitBulkCmd):
    """
    This is the same as the Flux SubmitCmd, but we add the host and port
    of the grpc service

    Usage: flux submit [OPTIONS] cmd ...
    """
    def __init__(self, prog, usage=None, description=None):
        super().__init__(prog, usage, description)
        self.parser.add_argument(
            "command", nargs=argparse.REMAINDER, help="Job command and arguments"
        )
        self.parser.add_argument(
            "--port",
            help=f"Port to run application (defaults to {defaults.port})",
            default=defaults.port,
            type=int,
        )
        self.parser.add_argument(
             "--host",
            help="Host to run application (defaults to localhost)",
            default="localhost",
        )

    def submit_async(self, args, jobspec=None):
        """
        Override submit async to always return the jobspec.
        """
        if jobspec is None:
            jobspec = self.jobspec_create(args)
        return jobspec.dumps()

    def submit(self, args, jobspec=None):
        return self.submit_async(args, jobspec)

def main():
    sys.stdout = open(
        sys.stdout.fileno(), "w", encoding="utf8", errors="surrogateescape"
    )
    sys.stderr = open(
        sys.stderr.fileno(), "w", encoding="utf8", errors="surrogateescape"
    )

    # Prepare the submit parser
    submit = SubmitCmd(
        "flux submit",
        description="enqueue a job",
    )
    parser = submit.get_parser()
    parser.set_defaults(func=submit.submit)
    args = parser.parse_args()

    # This is a json string
    jobspec = args.func(args)
    cli = FluxClient(f"{args.host}:{args.port}")
    response = cli.submit(jobspec)
    print(f"Submit response: {response.status}")


if __name__ == "__main__":
    main()




from fluxgrpc.protos import flux_pb2, flux_pb2_grpc
import fluxgrpc.auth as auth

class FluxClient:
    """
    The FluxClient is used to communicate with the grpc service.
    """

    def __init__(self, host="localhost:50051", use_ssl=False):
        self.host = host
        self.use_ssl = use_ssl

    def submit(self, jobspec):
        """
        Submit a job to flux via a json jobspec
        """
        # These are submit variables. A more substantial submit script would have argparse, etc.
        request = flux_pb2.SubmitRequest(jobspec=jobspec)

        with auth.grpc_channel(self.host, self.use_ssl) as channel:
            stub = flux_pb2_grpc.FluxServiceStub(channel)
            return stub.Submit(request)



#!/usr/bin/env python

# This is the ensemble client init, where we can start a client for a particular
# queue backend (like Flux)

import argparse
import sys

import fluxgrpc
import fluxgrpc.defaults as defaults


def get_parser():
    parser = argparse.ArgumentParser(
        description="Flux GRPC Python",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument(
        "--version",
        dest="version",
        help="show software version.",
        default=False,
        action="store_true",
    )

    description = "actions for Flux GRPC Python"
    subparsers = parser.add_subparsers(
        help="actions",
        title="actions",
        description=description,
        dest="command",
    )

    subparsers.add_parser("version", description="show software version")

    # Submit a job to flux
    submit = subparsers.add_parser(
        "submit",
        description="run the ensemble",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    submit.add_argument(
        "--env",
        help="Environment variable to include",
        action="add",
        default=[],
    )
    submit.add_argument(
        "-N"
        "--nodes",
        default=1,
        help="Number of nodes for the job",
    )
    submit.add_argument(
        "--debug",
        help="Enable debug logging for the config",
        action="store_true",
        default=False,
    )
    submit.add_argument(
        "--port",
        help=f"Port to run application (defaults to {defaults.port})",
        default=defaults.port,
        type=int,
    )
    submit.add_argument(
        "--host",
        help="Host to run application (defaults to localhost)",
        default="localhost",
    )
    return parser


def run_():
    parser = get_parser()

    def help(return_code=0):
        version = ensemble.__version__

        print("\nFlux Service v%s" % version)
        parser.print_help()
        sys.exit(return_code)

    # If the user didn't provide any arguments, show the full help
    if len(sys.argv) == 1:
        help()

    # If an error occurs while parsing the arguments, the interpreter will exit with value 2
    args, extra = parser.parse_known_args()

    # Show the version and exit
    if args.command == "version" or args.version:
        print(ensemble.__version__)
        sys.exit(0)

    # retrieve subparser (with help) from parser
    helper = None
    subparsers_actions = [
        action for action in parser._actions if isinstance(action, argparse._SubParsersAction)
    ]
    for subparsers_action in subparsers_actions:
        for choice, subparser in subparsers_action.choices.items():
            if choice == args.command:
                helper = subparser
                break

    # Does the user want a shell?
    if args.command == "run":
        from .run import main

    # Pass on to the correct parser
    return_code = 0
    try:
        main(args=args, parser=parser, extra=extra, subparser=helper)
        sys.exit(return_code)
    except UnboundLocalError:
        return_code = 1
    help(return_code)


if __name__ == "__main__":
    run_ensemble()



from contextlib import contextmanager

import grpc


@contextmanager
def grpc_channel(host, use_ssl=False):
    """
    Yield a channel, either with or without ssl, and close properly.
    """
    if use_ssl:
        channel = grpc.secure_channel(host, grpc.ssl_channel_credentials())
    else:
        channel = grpc.insecure_channel(host)
    try:
        yield channel
    finally:
        channel.close()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: flux.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'flux.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nflux.proto\x12\x04\x66lux\" \n\rSubmitRequest\x12\x0f\n\x07jobspec\x18\x01 \x01(\t\"\x88\x01\n\x0eSubmitResponse\x12/\n\x06status\x18\x01 \x01(\x0e\x32\x1f.flux.SubmitResponse.ResultType\"E\n\nResultType\x12\x12\n\x0eSUBMIT_SUCCESS\x10\x00\x12\x10\n\x0cSUBMIT_ERROR\x10\x01\x12\x11\n\rSUBMIT_DENIED\x10\x02\"\x1c\n\x0bInfoRequest\x12\r\n\x05jobid\x18\x01 \x01(\t\"~\n\x0cInfoResponse\x12-\n\x06status\x18\x01 \x01(\x0e\x32\x1d.flux.InfoResponse.ResultType\"?\n\nResultType\x12\x10\n\x0cINFO_SUCCESS\x10\x00\x12\x0e\n\nINFO_ERROR\x10\x01\x12\x0f\n\x0bINFO_DENIED\x10\x02\"\r\n\x0bJobsRequest\"(\n\x07JobInfo\x12\r\n\x05jobid\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t\"\x8c\x01\n\x0cJobsResponse\x12-\n\x06status\x18\x01 \x01(\x0e\x32\x1d.flux.JobsResponse.ResultType\x12\x0c\n\x04jobs\x18\x02 \x01(\t\"?\n\nResultType\x12\x10\n\x0cJOBS_SUCCESS\x10\x00\x12\x0e\n\nJOBS_ERROR\x10\x01\x12\x0f\n\x0bJOBS_DENIED\x10\x02\x32u\n\x0b\x46luxService\x12\x35\n\x06Submit\x12\x13.flux.SubmitRequest\x1a\x14.flux.SubmitResponse\"\x00\x12/\n\x04Jobs\x12\x11.flux.JobsRequest\x1a\x12.flux.JobsResponse\"\x00\x42\x31Z/github.com/flux-framework/flux-go/pkg/flux-grpcb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'flux_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z/github.com/flux-framework/flux-go/pkg/flux-grpc'
  _globals['_SUBMITREQUEST']._serialized_start=20
  _globals['_SUBMITREQUEST']._serialized_end=52
  _globals['_SUBMITRESPONSE']._serialized_start=55
  _globals['_SUBMITRESPONSE']._serialized_end=191
  _globals['_SUBMITRESPONSE_RESULTTYPE']._serialized_start=122
  _globals['_SUBMITRESPONSE_RESULTTYPE']._serialized_end=191
  _globals['_INFOREQUEST']._serialized_start=193
  _globals['_INFOREQUEST']._serialized_end=221
  _globals['_INFORESPONSE']._serialized_start=223
  _globals['_INFORESPONSE']._serialized_end=349
  _globals['_INFORESPONSE_RESULTTYPE']._serialized_start=286
  _globals['_INFORESPONSE_RESULTTYPE']._serialized_end=349
  _globals['_JOBSREQUEST']._serialized_start=351
  _globals['_JOBSREQUEST']._serialized_end=364
  _globals['_JOBINFO']._serialized_start=366
  _globals['_JOBINFO']._serialized_end=406
  _globals['_JOBSRESPONSE']._serialized_start=409
  _globals['_JOBSRESPONSE']._serialized_end=549
  _globals['_JOBSRESPONSE_RESULTTYPE']._serialized_start=486
  _globals['_JOBSRESPONSE_RESULTTYPE']._serialized_end=549
  _globals['_FLUXSERVICE']._serialized_start=551
  _globals['_FLUXSERVICE']._serialized_end=668
# @@protoc_insertion_point(module_scope)
